# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(440, 216)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.btn1 = QtWidgets.QPushButton(self.centralwidget)
        self.btn1.setGeometry(QtCore.QRect(360, 20, 71, 71))
        self.btn1.setObjectName("btn1")
        self.btn2 = QtWidgets.QPushButton(self.centralwidget)
        self.btn2.setGeometry(QtCore.QRect(360, 130, 71, 71))
        self.btn2.setObjectName("btn2")
        self.slider = QtWidgets.QSlider(self.centralwidget)
        self.slider.setGeometry(QtCore.QRect(20, 10, 22, 191))
        self.slider.setOrientation(QtCore.Qt.Vertical)
        self.slider.setObjectName("slider")
        self.led_lbl1 = QtWidgets.QLabel(self.centralwidget)
        self.led_lbl1.setGeometry(QtCore.QRect(50, 10, 101, 201))
        self.led_lbl1.setText("")
        self.led_lbl1.setPixmap(QtGui.QPixmap("ledoff.png"))
        self.led_lbl1.setObjectName("led_lbl1")
        self.led_lbl2 = QtWidgets.QLabel(self.centralwidget)
        self.led_lbl2.setGeometry(QtCore.QRect(150, 10, 101, 201))
        self.led_lbl2.setText("")
        self.led_lbl2.setPixmap(QtGui.QPixmap("ledoff.png"))
        self.led_lbl2.setObjectName("led_lbl2")
        self.led_lbl3 = QtWidgets.QLabel(self.centralwidget)
        self.led_lbl3.setGeometry(QtCore.QRect(260, 10, 101, 201))
        self.led_lbl3.setText("")
        self.led_lbl3.setPixmap(QtGui.QPixmap("ledoff.png"))
        self.led_lbl3.setObjectName("led_lbl3")
        MainWindow.setCentralWidget(self.centralwidget)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "IO Emulator"))
        self.btn1.setText(_translate("MainWindow", "1"))
        self.btn2.setText(_translate("MainWindow", "2"))




# ioemu/__init__.py
'''
Client and server of the emulator are communicating with the following protocol:

Request: xyz where x, y and z in (0,1) will turn the LED x, y or z  
  on(1) or off(0). Request with a length other than 3 will have no effect on 
  the status of the LEDs but can be used to force a response.

Response: lll;bb;aa where each l and b is in (0, 1) and represents the state
  of an LED or button. aa is the value of the analog slider between 00 and 99.
'''
import os
import sys
import socket
import pkg_resources

import PyQt5
import PyQt5.QtNetwork

from . import mainwindow

VERSION = pkg_resources.get_distribution('ioemu').version


# taken from https://openclipart.org/detail/248021/red-led-lamp-on
LED_ON_FILE = 'ledon.png'
# taken from https://openclipart.org/detail/248019/red-led-lamp
LED_OFF_FILE = 'ledoff.png'
# taken from https://openclipart.org/detail/299643/dpst-micro-push-button-switch
BUTTON_FILE = 'button.png'

TCP_SERVER_PORT = int(os.environ.get('IOEMU_PORT', default='9999'))
NUM_LEDS = 3  # don't change this unless you know what you are doing.


def request_compose(buttons):
    assert len(buttons) == 3

    req = ''
    for b in buttons:
        req += '1' if b else '0'

    return req

def request_decompose(payload):
    'Assume payload lll with boolean values.'
    assert len(payload) == 3

    leds = []
    for l in payload:
        leds.append(l == '1')

    return leds

def response_compose(leds, buttons, analog):
    'Create response of format lll;bb;aa'
    resp = ''
    for led in leds:
        resp += '1' if led else '0'
    resp += ';'

    for btn in buttons:
        resp += '1' if btn else '0'
    resp += ';'

    # add slider value to response
    resp += str(analog).zfill(2)
    return resp

def response_decompose(payload):
    '''decompose payload into three entries: LED states, button 
    states, analog value. Assuming format lll;bb;aa'''

    assert len(payload) == 9    
    lll, bb, aa = payload.split(';')
    leds = []
    for l in lll:
        leds.append(l == '1')

    buttons = []
    for b in bb:
        buttons.append(b == '1')

    analog = int(aa)

    return leds, buttons, analog



class EmulatorGui(mainwindow.Ui_MainWindow):

    def __init__(self, port=TCP_SERVER_PORT):
        super().__init__()

        self.button_pressed = [False, False]
        self.slider_value = 0

        self._ledon = PyQt5.QtGui.QPixmap(_absolute_path(LED_ON_FILE))
        self._ledoff = PyQt5.QtGui.QPixmap(_absolute_path(LED_OFF_FILE))

        self._buffer = 0

        self._tcp_server = PyQt5.QtNetwork.QTcpServer()
        self._tcp_server.newConnection.connect(self._new_session)

        self._tcp_server.listen(address=PyQt5.QtNetwork.QHostAddress.LocalHost,
                                port=port)       

    def setupUi(self, main_win):
        super().setupUi(main_win)

        # connection slots
        self.btn1.pressed.connect(lambda: self._btn_clicked(0, True))
        self.btn2.pressed.connect(lambda: self._btn_clicked(1, True))
        self.btn1.released.connect(lambda: self._btn_clicked(0, False))
        self.btn2.released.connect(lambda: self._btn_clicked(1, False))
        self.slider.valueChanged.connect(self._slider_value_changed)

        # setting icons
        pix = PyQt5.QtGui.QPixmap(_absolute_path(BUTTON_FILE))
        buttonpic = PyQt5.QtGui.QIcon(pix)
        self.btn1.setIcon(buttonpic)
        self.btn1.setIconSize(self.btn1.rect().size())
        self.btn2.setIcon(buttonpic)
        self.btn2.setIconSize(self.btn2.rect().size())

        self.led_lbl1.setPixmap(self._ledoff)
        self.led_lbl2.setPixmap(self._ledoff)
        self.led_lbl3.setPixmap(self._ledoff)
        self._led_lbls = [self.led_lbl1, self.led_lbl2, self.led_lbl3]

    def _new_session(self):
        sock = self._tcp_server.nextPendingConnection()
        sock.waitForReadyRead(msecs=100)

        # assuming 010, 110, ...
        payload = str(sock.readAll(), "ASCII").strip()
        #print("request", payload)
        if len(payload) == 3:
            # update screen
            self.write(int(payload, base=2))

        # create response: state of buttons
        leds = self.__get_led_state()
        resp = response_compose(leds, self.button_pressed, self.slider_value)
        sock.write(bytes(resp, "ASCII"))
        sock.disconnectFromHost()

    def __get_led_state(self):
        'Read the LEDs state from the pixmap of the labels.'
        return [
            self.led_lbl1.pixmap() == self._ledon,
            self.led_lbl2.pixmap() == self._ledon,
            self.led_lbl3.pixmap() == self._ledon
        ]

    def _slider_value_changed(self):
        self.slider_value = self.slider.value()

    def _btn_clicked(self, num, on_off):
        self.button_pressed[num] = on_off

    def write(self, buffer):
        '''Write value to display buffer. Buffer must be an integer whose 
        binary representation will be shown on the display.'''
        assert 0 <= buffer < 2 ** NUM_LEDS, buffer

        self._buffer = buffer
        self._update_screen()

    def _update_screen(self):
        payload_str = bin(self._buffer)[2:]
        payload_str = payload_str.zfill(NUM_LEDS)
        
        # draw LEDs
        for i in range(len(payload_str)):            
            image = self._ledon if payload_str[i] == '1' else self._ledoff
            self._led_lbls[i].setPixmap(image)


class Emulator:

    def __init__(self, port=TCP_SERVER_PORT):
        self.host = 'localhost'
        self.port = port
        self.sock = socket.socket()

    @property
    def buttons(self):
        _, buttons, _ = self._send(' ')
        return buttons

    @property
    def analog_value(self):
        # response has format xx.yy where yy is the analog value
        _, _, analog = self._send(' ')
        return analog

    @property
    def leds(self):
        led, _, _ = self._send(' ')
        return led

    @leds.setter
    def leds(self, abc):
        assert len(abc) == 3, "There must be three values for the LEDs."

        payload = request_compose(abc)

        #print("sending", payload)
        _, buttons, _ = self._send(payload)
        self.button_pressed = buttons

    def _send(self, payload):
        'send payload and return response.'
        with socket.socket() as sock:
            try:
                sock.connect((self.host, self.port))
            except ConnectionRefusedError:
                raise Exception("Unable to connect to Emulator. Maybe it is not running.")
            sock.send(bytes(payload, 'ASCII'))
            response = str(sock.recv(32), 'ASCII')
            #print("response", response)

            return response_decompose(response)


def _absolute_path(filename):
    'Create absolute path for given filename.'
    return os.path.join(os.path.dirname(__file__), filename)
        

def run():
    print("ioemu (version %s)" % VERSION)
    print("listening on localhost port", TCP_SERVER_PORT)
    app = PyQt5.QtWidgets.QApplication([])
    main_win = PyQt5.QtWidgets.QMainWindow()
    emu = EmulatorGui()
    emu.setupUi(main_win)
    main_win.show()
    emu.write(0b101)

    app.exec_()



import os
import time

import ioemu

# Port to which the demo client talks
# Can be overridden by an environmental variable.
DEMO_CLIENT_PORT = int(os.environ.get('IOEMU_DEMO_CLIENT_PORT', default=ioemu.TCP_SERVER_PORT))


emu = ioemu.Emulator(port=DEMO_CLIENT_PORT)
last_btn_state = emu.buttons
last_analog_val = emu.analog_value
last_update = time.time()
counter = 0

while True:
    if time.time() - last_update > 0.4:
        emu.leds = [b == '1' for b in bin(counter)[2:].zfill(3)]
        counter = (counter + 1) % 8
        last_update = time.time()

    if emu.buttons != last_btn_state:
        last_btn_state = emu.buttons
        print("Buttons:", last_btn_state)

    if emu.analog_value != last_analog_val:
        last_analog_val = emu.analog_value
        print("Analog Value:", last_analog_val)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ioemu']

package_data = \
{'': ['*']}

install_requires = \
['PyQt5>=5.11.3,<6.0.0']

entry_points = \
{'console_scripts': ['ioemu = ioemu:run']}

setup_kwargs = {
    'name': 'ioemu',
    'version': '0.3.1',
    'description': 'IO Emulator with LEDs and buttons',
    'long_description': "# ioemu\n\n![screenshot](ioemu-screenshot.png)\n\nThe ioemu-project provides an  emulator for input/output operations with simple electronic components like LEDs and push buttons.\n\n## Installation and Upgrade\n\nUse pip for a simple installation. For an update use `install --upgrade`. \n\n- Linux, MacOS: `python3 -m pip install ioemu`\n- Windows: `python -m pip install ioemu`\n\n## Starting the emulator\n\nFirst start the emulator by entering `ioemu` on the commandline. A Gui will show up.\n\n![screenshot](ioemu-screenshot.png)\n\nIt contains a slider for analog values between 0 and 99, threee LEDs and two push buttons from left to right.\n\n## LEDs\n\nIf the emulator is running, you can interact with it from any python program running on the same machine. First import the class `Emulator` from the `ioemu` package.\n\n\n```python\nfrom ioemu import Emulator\n```\n\nNow create an instance of the emulator and switch some LEDs on. They can be controlled by setting the `leds` attribute.\n\n\n```python\nemu = Emulator()\nemu.leds = [True, False, True]\n```\n\n## Buttons\n\n![screenshot](buttons.gif)\n\nThe emulator has two buttons. Their current state (pressed or not pressed) can be read from the attribute `buttons`. It's a bool array corresponding to the state of being pressed.\n\nThe following program lights up some LEDs depending on the button being pressed.\n\n\n```python\nemu = Emulator()\nwhile True:\n    if emu.buttons[0]:\n        emu.leds = [False, True, True]\n        \n    if emu.buttons[1]:\n        emu.leds = [True, True, False]\n\n    if not (emu.buttons[0] or emu.buttons[1]):\n        emu.leds = [False, False, False]\n```\n\n## Analog Value (0-99)\n\nLet's look into a program that allows you to control the LEDs with the slider at the left. The current sliders value can be read from the `analog_value` attribute of the Emulator. Its value ranges from 0 to 99.\n\n![image](analog_value.gif)\n\n\n```python\nimport time\n\nemu = Emulator()\nled_on = 0\n\nwhile True:\n    if 0 <= emu.analog_value < 25:\n        emu.leds = [False, False, False]\n    elif 25 <= emu.analog_value < 50:\n        emu.leds = [True, False, False]\n    elif 50 <= emu.analog_value < 75:\n        emu.leds = [True, True, False]\n    else:\n        emu.leds = [True, True, True]\n```\n\n## Demo\n\nThere is a demo program that can be started with `python -m ioemu.demo`. It will blink the LEDs and print the current button state as well as the analog value to console.\n\n![demo](demo.gif)\n\nYou can find the source code in [demo.py](ioemu/demo.py).\n\n## Bugs\n\nIf you find any bugs or have a feature request, feel free to file a ticket at the projects [bugtracker](https://github.com/tbs1-bo/ioemu/issues/new) on github.\n",
    'author': 'Marco Bakera',
    'author_email': 'marco@bakera.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/tbs1-bo/ioemu',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
